import matplotlib.pyplot as plt
from numpy import exp, linspace, inf
from scipy.optimize import curve_fit
import scienceplots
plt.style.use(["science", "grid"])

def sigmoid(x, right, curve):
    return 1 - 1/(1 + exp(right-curve*x))

col = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22']
marker = ["v", "p", "h", "8", "s", 'P', 'D', 'X', "*"]

def plot_lll(filepath='results_lll.txt'):
    d = open(filepath, 'r')
    dt = d.readlines()
    n = []
    q = []
    x = []
    for l in dt:
        ll = l.split()
        q.append(int(ll[0]))
        n.append(int(ll[1]))
        x.append(float(ll[2]))
    k=0
    x_val = linspace(1, 80, 1001)
    unique_q = list(set(q))
    unique_q.sort()
    s, q_s = [], []
    plt.figure(figsize=(4,4))
    for qs in unique_q:
        indicies = [i for i in range(len(q)) if q[i] == qs]
        nn = [n[i] for i in indicies]
        xx = [x[i] for i in indicies]
        assert len(nn) == len(xx)
        popt, pcov  = curve_fit(sigmoid, nn,xx, bounds=[(-20, 0), (120, inf)])
        q_s.append(qs)
        s.append(popt[0]/popt[1])
        if qs in [71, 401, 3329]:
            plt.plot(nn, xx, marker=marker[k%3], linestyle='', markersize=10,
                                   color=col[k%3],
                                   markerfacecolor=col[k%3],
                                   markerfacecoloralt=col[k%3],
                                   markeredgecolor=col[k%3], fillstyle='none', label=f"q={qs}")

            yy = sigmoid(x_val, popt[0], popt[1])
            plt.plot(x_val, yy, '--', color=col[k%3])
            k += 1
    plt.xlabel(r'Key length $n$', fontsize=12)
    plt.ylabel(r'Probability $p$', fontsize=12)
    legend = plt.legend(fancybox=False, edgecolor="black", loc=3)
    legend.get_frame().set_linewidth(0.5)

    plt.figure()
    plt.plot(q_s[1:], s[1:], marker=marker[1], linestyle='', markersize=10, color=col[0], fillstyle='none',
                markerfacecolor=col[0],
                markerfacecoloralt=col[0],
                markeredgecolor=col[0])
    plt.xlabel(r'Modulus $q$', fontsize=12)
    plt.ylabel(r'Key length $n$', fontsize=12)
    plt.show()


def plot_bkz(filename='bkz_results.txt', show_time=False, prime_for_time=401):
    d = open(filename, 'r')
    nn, pp, bb, score, time, sample = [], [], [], [], [], []
    l = d.readline()
    while l != '':
        l = l.replace('\n', '')
        strg = l.split(' ')
        nn.append(int(strg[0]))
        pp.append(int(strg[1]))
        bb.append(int(strg[2]))
        score.append(float(strg[3]))
        if strg[4] == 'nan':
            time.append(-1.0)
            sample.append(-1)
        else:
            time.append(float(strg[4]))
            sample.append(int(strg[5]))
        l = d.readline()
    assert len(nn) == len(pp) == len(bb) == len(score) == len(time) == len(sample)
    n = list(set(nn))
    n.sort()
    p = set(pp)
    b = list(set(bb))
    b.sort()
    results = dict()
    for block in b:
        results[block] = dict()
        for module in p:
            results[block][module] = dict()
            for keylength in n:
                results[block][module][keylength] = dict()
                idx = [i for i in range(len(nn)) if nn[i] == keylength and pp[i] == module and bb[i] == block]
                if len(idx) == 1:
                    results[block][module][keylength]['score'] = score[idx[0]]
                    results[block][module][keylength]['time'] = time[idx[0]]
                    results[block][module][keylength]['samplesize'] = sample[idx[0]]
                elif len(idx) > 1:
                    z = len(idx)
                    samples = [sample[i] for i in idx]
                    times = [time[i] for i in idx]
                    scores = [score[i] for i in idx]
                    weighted_times = sum([times[i]*samples[i] for i in range(z)])/sum(samples)
                    weighted_scores = sum([scores[i]*samples[i] for i in range(z)])/sum(samples)
                    try:
                        results[block][module][keylength]['score'] = weighted_scores
                        results[block][module][keylength]['time'] = weighted_times
                        results[block][module][keylength]['samplesize'] = sum(samples)
                    except:
                        print(block, module, keylength)
    block_plot = list(set(bb))
    block_plot.sort()
    block_plot.remove(2)
    primes = list(set(pp))
    primes.sort()
    k = 0
    for prime in primes:
        plt.figure(figsize=(4,4))
        for block in block_plot:
            nnn = []
            scores = []
            for keylength in n:
                nnn.append(keylength)
                scores.append(results[block][prime][keylength]['score'])
            plt.plot(nnn, scores, marker=marker[k], linestyle='-', markersize=5,
                     color=col[k],
                     markerfacecolor=col[k],
                     markerfacecoloralt=col[k],
                     markeredgecolor=col[k], fillstyle='none', label=fr"$\beta={block}$")
            k += 1
            k %= 9
        legend = plt.legend(fancybox=False, edgecolor="black", loc=3)
        legend.get_frame().set_linewidth(0.5)
        plt.xlabel(r'Key length $n$', fontsize=12)
        plt.ylabel(r'Probability $p$', fontsize=12)
    if show_time:
        plt.figure(figsize=(6,4))
        k = 0
        for keylength in n:
            if keylength > 40:
                b = []
                times = []
                for blocks in block_plot:
                    b.append(blocks)
                    times.append(results[blocks][prime_for_time][keylength]['time'])
                plt.plot(b, times, marker=marker[k], linestyle='-', markersize=5, color=col[k], fillstyle='none', label=fr"$n$ = {keylength}")

                k += 1
        legend = plt.legend(fancybox=False, edgecolor="black", bbox_to_anchor=(1.05, 1.0), loc='upper left')
        legend.get_frame().set_linewidth(0.5)
        plt.tight_layout()
        plt.xlabel(r'Block size $\beta$', fontsize=12)
        plt.ylabel(r'Time $t\: [s]$', fontsize=12)
        plt.yscale('log')
        plt.show()
